package com.ruoyi.bst.fault.service.impl;

import com.ruoyi.bst.area.domain.AreaVO;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.service.DeviceService;
import com.ruoyi.bst.fault.domain.Fault;
import com.ruoyi.bst.fault.domain.FaultQuery;
import com.ruoyi.bst.fault.domain.FaultVO;
import com.ruoyi.bst.fault.domain.enums.FaultStatus;
import com.ruoyi.bst.fault.service.FaultConverter;
import com.ruoyi.bst.fault.service.FaultService;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.service.OrderService;
import com.ruoyi.common.core.domain.vo.UserVO;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.system.user.service.UserService;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

@Service
public class AbnormalConverterImpl implements FaultConverter {

    @Autowired
    private FaultService faultService;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private UserService userService;

    @Override
    public Fault toPo(FaultVO fault) {
        DeviceVO device = deviceService.selectDeviceBySn(fault.getVehicleCode());
        ServiceUtil.assertion(device == null, "设备不存在");

        Fault po = new Fault();
        po.setFaultSite(fault.getFaultSite());
        po.setVehicleCode(fault.getVehicleCode());
        po.setPicture(fault.getPicture());
        po.setFaultDetail(fault.getFaultDetail());
        po.setCreateBy(fault.getCreateBy());
        po.setAppealStatus(FaultStatus.PENGDING.getCode());
        po.setCreateTime(DateUtils.getNowDate());
        po.setUserName(fault.getCreateBy());
        po.setUserId(fault.getUserId());
        // 如果订单存在，存入订单时间和订单结束时间
        if (fault.getOrderId() != null) {
            OrderVO order = orderService.selectOrderById(fault.getOrderId());
            ServiceUtil.assertion(order == null,"订单不存在");
            Long orderId = order.getId();
            if (device.getOrderId() != null && orderId.equals(device.getOrderId())) {
                po.setOrderId(orderId);
                if (order.getEndTime() != null) {
                    po.setOrderEndTime(DateUtils.toDate(order.getEndTime()));
                }
        }
        }
        String vehicleNum = device.getVehicleNum();
        if (vehicleNum != null) {
            po.setVehicleNum(vehicleNum);
        }
        Long mchId = device.getMchId();
        if (mchId != null) {
            po.setStoreId(mchId);
        }
        return po;
    }

    @Override
    public boolean canEdit(Long faultId) {
        return canOperate(Arrays.asList(faultId));
    }

    private boolean canOperate(List<Long> faultIds) {
        if (SecurityUtils.isSysAdmin()){
            return true;
        }

        FaultQuery query = new FaultQuery();
        query.setId(faultIds.get(0));
        query.setScope(true);
        List<FaultVO> faultList = faultService.selectFaultList(query);
        Long userId = SecurityUtils.getUserId();

        // 判断是否可以处理故障
        for (Long faultId : faultIds) {
            FaultVO fault = faultList.stream()
                    .filter(item -> item.getId().equals(faultId))
                    .findFirst().orElse(null);
            if (fault == null) {
                return false;
            }
            boolean canOperate = isStore(fault, userId);
            if (!canOperate) {
                return false;
            }
        }
        return true;
    }

    private boolean isStore(FaultVO fault, Long userId) {
        return fault != null && fault.getStoreId() != null && fault.getStoreId().equals(userId);
    }
}
